# Not necessarily tests for various.jl

@testset "linalg" begin
    s = TSeries(20Q1, collect(10.0 .+ (1:12)))
    s2 = TSeries(20Q1, [2])
    s3 = TSeries(20Q1, collect(10.0 .+ (1:10)))
    x = MVTSeries(1U:10U, (:a, :b))
    a = collect(1:10)
    b = collect(11:20)
    x.a = a
    x.b = b

    x2 = MVTSeries(1U:10U, (:a, :b))
    x2.a = a
    x2.b = b

    x3 = MVTSeries(1U:2U, (:a, :b, :c, :d, :e, :f, :g, :h, :i, :j))
    x3.a = collect(1:2)
    x3.b = collect(1:2)
    x3.c = collect(1:2)
    x3.d = collect(1:2)
    x3.e = collect(1:2)
    x3.f = collect(1:2)
    x3.g = collect(1:2)
    x3.h = collect(1:2)
    x3.i = collect(1:2)
    x3.j = collect(1:2)

    x4 = MVTSeries(1U:1U, (:a, :b, :c, :d, :e, :f, :g, :h, :i, :j))
    x4.a = collect(2:2)
    x4.b = collect(2:2)
    x4.c = collect(2:2)
    x4.d = collect(2:2)
    x4.e = collect(2:2)
    x4.f = collect(2:2)
    x4.g = collect(2:2)
    x4.h = collect(2:2)
    x4.i = collect(2:2)
    x4.j = collect(2:2)


    @test adjoint(s) ≈ reshape(collect(10.0 .+ (1:12)), (1, 12))
    @test adjoint(x) ≈ [a'; b']

    @test x / x2 == TimeSeriesEcon._vals(x) / TimeSeriesEcon._vals(x2)
    @test x / TimeSeriesEcon._vals(x2) == TimeSeriesEcon._vals(x) / TimeSeriesEcon._vals(x2)
    @test TimeSeriesEcon._vals(x) / x2 == TimeSeriesEcon._vals(x) / TimeSeriesEcon._vals(x2)
    @test x * x3 == TimeSeriesEcon._vals(x) * TimeSeriesEcon._vals(x3)
    @test x \ x2 == TimeSeriesEcon._vals(x) \ TimeSeriesEcon._vals(x2)

    @test s * x4 == TimeSeriesEcon._vals(s) * TimeSeriesEcon._vals(x4)
    @test x4 * s3 == TimeSeriesEcon._vals(x4) * TimeSeriesEcon._vals(s3)
    @test s / s2 == TimeSeriesEcon._vals(s) / TimeSeriesEcon._vals(s2)
end


@testset "findall" begin
    # findall works for TSeries
    tt = TSeries(2000Q1, rand(10))
    tb = tt .> 0.5
    @test findall(tb) isa Vector{<:MIT}
    @test length(findall(tb)) == sum(tb)
    @test findall(tb) == rangeof(tb)[findall(tb.values)]
    # findall works for MVTSeries
    tv = MVTSeries(2000Q1, (:a, :b, :c), rand(10, 3))
    tm = tv .> 0.5
    @test findall(tm) isa Vector{CartesianIndex{2}}
    @test length(findall(tm)) == sum(tm)

    # getindex works with TSeries of Bool
    @test tt.values[tb.values] == tt[tb]
    # setindex works with TSeries of Bool
    @test (tt[tb] = 0.1 .+ (1:sum(tb)); (tt .> 1) == tb)
    # broadcasting works with TSeries of Bool
    @test (tt[tb] .= -1.0; tb == (tt .< 0.0))

    # getindex works with MVTSeries of Bool
    @test tv.values[tm.values] == tv[tm]
    # setindex works with MVTSeries of Bool
    @test (tv[tm] = 0.1 .+ (1:sum(tm)); (tv .> 1) == tm)
    # broadcasting works with MVTSeries of Bool
    @test (tv[tm] .= -1.0; tm == (tv .< 0.0))

    @test (tv[tb] == tv.values[tb.values, :])
    @test (tv[tb, :] == tv.values[tb.values, :])
    @test (tv[tb] = -1000 * ones(sum(tb), 3); sum(tv[tb]) == -1000 * 3 * sum(tb))
    @test (tv[tb, :] = -2000 * ones(sum(tb), 3); sum(tv[tb]) == -2000 * 3 * sum(tb))
    @test (tv[tb] .= -3000; sum(tv[tb]) == -3000 * 3 * sum(tb))
    @test (tv[tb, :] .= -4000; sum(tv[tb]) == -4000 * 3 * sum(tb))


end

@testset "overlay2" begin
    a = MVTSeries(20Q1, (:a, :b, :c), ones(7, 3))
    b = MVTSeries(21Q1, (:q, :b, :c, :f), 10 .+ 0.1 * ones(10, 4))
    c = overlay(a, b)
    @test c isa MVTSeries
    @test frequencyof(c) == frequencyof(b) == frequencyof(a)
    @test rangeof(c) == 20Q1:23Q2
    @test axes(c, 2) == [:a, :b, :c, :q, :f]
    mat1 = [
        1.0 1.0 1.0 NaN NaN
        1.0 1.0 1.0 NaN NaN
        1.0 1.0 1.0 NaN NaN
        1.0 1.0 1.0 NaN NaN
        1.0 1.0 1.0 10.1 10.1
        1.0 1.0 1.0 10.1 10.1
        1.0 1.0 1.0 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1
        NaN 10.1 10.1 10.1 10.1]
    @test isapprox(c, mat1, nans=true)
    @test isapprox(mat1, c, nans=true)
    @test isapprox(c.a, mat1[:, 1], nans=true)
    @test isapprox(mat1[:, 1], c.a, nans=true)
    d = overlay(b, a)
    @test c isa MVTSeries
    @test frequencyof(d) == frequencyof(b) == frequencyof(a)
    @test rangeof(d) == 20Q1:23Q2
    @test axes(d, 2) == [:q, :b, :c, :f, :a]
    @test isapprox(d, [
            NaN 1.0 1.0 NaN 1.0
            NaN 1.0 1.0 NaN 1.0
            NaN 1.0 1.0 NaN 1.0
            NaN 1.0 1.0 NaN 1.0
            10.1 10.1 10.1 10.1 1.0
            10.1 10.1 10.1 10.1 1.0
            10.1 10.1 10.1 10.1 1.0
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN
            10.1 10.1 10.1 10.1 NaN], nans=true)
end

@testset "misc" begin

    o = ones(20, 3)
    X = MVTSeries(2000Y, (:x, :y, :z), o)

    @test parent(X) === o
    @test parent(X.x) === view(o, :, 1)

    @test transpose(X) === transpose(o)
    @test transpose(X.x) === transpose(view(o, :, 1))

    Z = MVTSeries(2000Y, (:x, :y, :z), ones(15, 3))

    @test compare(X.values, Z.values, quiet=true) == false
    @test compare(X, Z, quiet=true, ignoremissing=false) == false
    @test compare(X, Z, quiet=true, ignoremissing=true) == true

    @test Base.axes1(X) === axes(X,1)

end
